# -*- coding: utf-8 -*-
from Products.CMFPlone.utils import safe_unicode
from kitconcept.glossary.interfaces import IGlossary
from kitconcept.glossary.interfaces import IGlossaryTerm
from plone.dexterity.content import Container
from plone.dexterity.content import Item
from zope.interface import implementer

import lxml


@implementer(IGlossary)
class Glossary(Container):

    """A Glossary is a container for Terms."""


@implementer(IGlossaryTerm)
class GlossaryTerm(Item):
    def Description(self):
        if not self.definition:
            return ""
        tree = lxml.html.fromstring(u"<div>%s</div>" % self.definition.output)
        text = tree.text_content()
        # Remove lxml ElementUnicodeResult which is a subclass of unicode
        text = safe_unicode(text.encode('utf-8'))
        return text



# -*- coding: utf-8 -*-
from zope.globalrequest import getRequest


def get_request_information():
    try:
        request = getRequest()
        user = request["AUTHENTICATED_USER"]
        ip = request["REMOTE_ADDR"]
        return user, ip
    except KeyError:
        return "test", "127.0.0.1"  # running tests



# -*- coding: utf-8 -*-
from kitconcept.glossary.interfaces import IGlossaryTerm
from plone.app.textfield.interfaces import ITransformer
from plone.i18n.normalizer.base import baseNormalize
from plone.indexer import indexer
from Products.CMFPlone.utils import safe_unicode


@indexer(IGlossaryTerm)
def textIndexer(context):
    """SearchableText contains id, title, variants and definition
    text as plain text.
    """
    transformer = ITransformer(context)

    try:
        definition = transformer(context.definition, "text/plain")
    except AttributeError:
        definition = u""
    try:
        variants = u" ".join(context.variants)
    except TypeError:
        variants = u""

    return u" ".join(
        (
            safe_unicode(context.id),
            safe_unicode(context.title) or u"",
            safe_unicode(variants),
            safe_unicode(definition),
        )
    )


@indexer(IGlossaryTerm)
def variantsIndexer(context):
    variants = context.variants or []
    return [context.title] + list(variants)


@indexer(IGlossaryTerm)
def definitionIndexer(context):
    return context.definition.raw


@indexer(IGlossaryTerm)
def letterIndexer(context):
    terms = []
    if context.title:
        terms.append(context.title)
    if context.variants:
        terms.extend(list(context.variants))
    normalized = [baseNormalize(term)[0].upper() for term in terms if term]
    normalized = list(set(normalized))
    return normalized



# -*- coding: utf-8 -*-
from plone import api
from plone.app.layout.viewlets import ViewletBase
from plone.i18n.normalizer.base import baseNormalize
from plone.memoize.instance import memoize
from Products.CMFPlone.PloneBatch import Batch
from Products.CMFPlone.utils import safe_unicode
from Products.Five.browser import BrowserView
from six import PY2
from zExceptions import Redirect

import itertools
import string

from kitconcept.glossary import _
from kitconcept.glossary.interfaces import IGlossarySettings

BATCH_SIZE = 30


def cmp(a, b):
    # cmp does not exist in py3 so we define it here
    return (a > b) - (a < b)


def _catalog_counter_cachekey(method, self):
    """Return a cachekey based on catalog updates."""

    catalog = api.portal.get_tool("portal_catalog")
    return str(catalog.getCounter())


class TermView(BrowserView):

    """Default view for Term type"""

    def get_entry(self):
        """Get term in the desired format"""

        item = {
            "term": self.context.title,
            "variants": self.context.variants,
            "definition": self.context.definition.raw,
        }
        return item


class GlossaryView(BrowserView):

    """Default view of Glossary type"""

    def __init__(self, context, request):
        super(GlossaryView, self).__init__(context, request)
        self.search_letter = request.get("search_letter", "")
        self.search_text = request.get("search_text")
        self.batch_start = request.get("b_start", 0)
        self.uid = context.UID()

    def title(self):
        """Title of our glossary"""

        return self.context.title_or_id()

    def first_letters(self):
        """Users with non latin chars (cyrillic, arabic, ...) should override
        this with a better suited dataset."""
        glossary_url = self.context.absolute_url()
        out = [{"glyph": _("All"),
                "has_no_term": False,
                "zoom_link": glossary_url,
                "css_class": not self.search_letter and "selected" or None,
                }]

        exists = any(
            [
                brain.letter
                for brain in api.content.find(
                    context=self.context,
                    depth=1,
                    portal_type="GlossaryTerm",
                    letter=tuple(string.digits),
                    sort_limit=1,
                )
            ]
        )
        letter_map = {
            "glyph": _("[0-9]"),
            "has_no_term": not exists,
            "zoom_link": glossary_url + "?search_letter=[0-9]",
            "css_class": (
                "[0-9]" == self.search_letter and "selected" or None
            ),
        }
        out.append(letter_map)

        for letter in tuple(string.ascii_uppercase):
            exists = any(
                [
                    brain.letter
                    for brain in api.content.find(
                        context=self.context,
                        depth=1,
                        portal_type="GlossaryTerm",
                        letter=letter,
                        sort_limit=1,
                    )
                ]
            )
            letter_map = {
                "glyph": letter,
                "has_no_term": not exists,
                "zoom_link": glossary_url + "?search_letter=" + letter.lower(),
                "css_class": (
                    letter.lower() == self.search_letter.lower() and "selected" or None
                ),
            }
            out.append(letter_map)
        return out

    def has_results(self):
        """Something to show ?"""

        return len(self._list_results()) > 0

    def batch_results(self):
        """Wrap all results in a batch"""

        results = self._list_results()
        batch = Batch(results, BATCH_SIZE, int(self.batch_start), orphan=1)
        return batch

    @memoize
    def _list_results(self):
        """Terms list (brains) depending on the request"""

        search_letter = self.search_letter
        if search_letter:
            search_letter = search_letter.upper()

        if search_letter == '[0-9]':
            search_letter = tuple(string.digits)

        common = {

            "context": self.context,
            "depth": 1,
            "portal_type": "GlossaryTerm",
        }

        if search_letter:
            results = api.content.find(
                letter=search_letter, **common)
        elif self.search_text:
            results = api.content.find(
                SearchableText=self.search_text, **common)
            # We redirect to the result if unique
            if len(results) == 1:
                target = results[0].getURL()
                raise Redirect(target)
        else:
            # Viewing all terms
            results = api.content.find(**common)
        results = list(results)
        variant_results = []
        # create a list of tuples with the sort key as the first item
        for brain in results:
            for variant in brain['variants']:
                sortable_variant = baseNormalize(variant.upper())
                if search_letter \
                   and isinstance(search_letter, str) \
                   and sortable_variant[0] != search_letter:
                    continue
                if search_letter \
                   and isinstance(search_letter, tuple) \
                   and sortable_variant[0] not in search_letter:
                    continue
                variant_results.append((sortable_variant,
                                        {
                                            "title": variant,
                                            "brain": brain,
                                            "letter": sortable_variant[0],
                                        }))
        variant_results = sorted(variant_results, key=lambda r: r[0])
        return tuple([r[1] for r in variant_results])

    def group_results_by_letter(self, results):
        grouped_results = itertools.groupby(results,
                                            lambda r: r['letter'])
        results = [{'letter': key, 'results': list(values)}
                   for (key, values)
                   in grouped_results]
        return results

    def truncateDescription(self, text):
        """Truncate definition using tool properties"""

        max_length = api.portal.get_registry_record(
            name="description_length", interface=IGlossarySettings,
        )
        ellipsis = api.portal.get_registry_record(
            name="description_limiter", interface=IGlossarySettings,
        )

        text = safe_unicode(text).strip()

        if max_length > 0 and len(text) > max_length:
            text = text[:max_length]
            text = text.strip()
            text = u"{0}{1}".format(text, ellipsis)

        if PY2:
            text = text.encode("utf-8", "replace")

        return text

    def result_features(self, result):
        """TAL friendly properties of each feature"""

        description = self.truncateDescription(result['brain'].Description)
        return {
            "url": result['brain'].getURL(),
            "title": result['title'] or result['brain'].getId,
            "description": description.replace("\n", "<br />"),
            "letter": result['letter'],
        }


class ResourcesViewlet(ViewletBase):
    """This viewlet inserts static resources on page header."""



# -*- coding: utf-8 -*-
from kitconcept.glossary.config import PROJECTNAME
from plone import api


def uninstall(portal, reinstall=False):
    if not reinstall:
        profile = "profile-{0}:uninstall".format(PROJECTNAME)
        setup_tool = api.portal.get_tool("portal_setup")
        setup_tool.runAllImportStepsFromProfile(profile)
        return "Ran all uninstall steps."



# -*- coding: utf-8 -*-
from Products.CMFPlone.interfaces import INonInstallable
from zope.interface import implementer


@implementer(INonInstallable)
class HiddenProfiles(object):  # pragma: no cover
    @staticmethod
    def getNonInstallableProfiles():
        """Do not show on Plone's list of installable profiles."""
        return [
            u"kitconcept.glossary:uninstall",
        ]



# -*- coding: utf-8 -*-
from kitconcept.glossary import _
from kitconcept.glossary.interfaces import IGlossarySettings
from plone.app.registry.browser import controlpanel


class GlossarySettingsEditForm(controlpanel.RegistryEditForm):

    """Control panel edit form."""

    schema = IGlossarySettings
    label = _(u"Glossary")
    description = _(u"Settings for the kitconcept.glossary package")


class GlossarySettingsControlPanel(controlpanel.ControlPanelFormWrapper):

    """Control panel form wrapper."""

    form = GlossarySettingsEditForm



# -*- coding: utf-8 -*-
from kitconcept.glossary import _
from kitconcept.glossary.config import DEFAULT_ENABLED_CONTENT_TYPES
from plone.app.textfield import RichText
from plone.autoform import directives as form
from plone.supermodel import model
from z3c.form.interfaces import IAddForm
from z3c.form.interfaces import IEditForm
from zope import schema
from zope.interface import Interface


class IGlossaryLayer(Interface):

    """A layer specific for this add-on product."""


class IGlossarySettings(Interface):

    """Schema for the control panel form."""

    enable_tooltip = schema.Bool(
        title=_(u"Enable tooltip?"), description=_(u"Enable tooltip."), default=True,
    )

    enabled_content_types = schema.List(
        title=_(u"Enabled Content Types"),
        description=_(
            u"Only objects of these content types will display " u"glossary terms."
        ),
        required=False,
        default=DEFAULT_ENABLED_CONTENT_TYPES,
        # we are going to list only the main content types in the widget
        value_type=schema.Choice(vocabulary=u"kitconcept.glossary.PortalTypes"),
    )

    description_length = schema.Int(
        title=_(u"Description length"), required=True, default=100,
    )

    description_limiter = schema.TextLine(
        title=_(u"Description ellipsis"), required=True, default=u"…",
    )


class IGlossary(Interface):

    """Container for GlossaryTerms."""

    text = RichText(title=_(u"Body text"), required=False,)


class IGlossaryTerm(Interface):

    title = schema.TextLine(title=_(u"Glossary Term"), required=True,)

    form.widget("variants", cols=25, rows=10)
    variants = schema.Tuple(
        title=_(u"Variants"),
        description=_(u"Enter the variants of the term, one per line."),
        required=False,
        value_type=schema.TextLine(),
        missing_value=(),
    )

    definition = RichText(
        title=_(u"Body text"), description=_(u"Enter the body text."), required=False,
    )

    model.fieldset("settings", label=_(u"Settings"), fields=["exclude_from_nav"])

    # https://community.plone.org/t/how-to-change-existing-dexterity-types-and-behaviors/219/6
    exclude_from_nav = schema.Bool(
        title=_(u"label_exclude_from_nav", default=u"Exclude from navigation"),
        description=_(
            u"help_exclude_from_nav",
            default=u"If selected, this item will not appear in the "
            u"navigation tree",
        ),
        default=True,  # Need to be True
    )

    form.omitted("exclude_from_nav")
    form.no_omit(IEditForm, "exclude_from_nav")
    form.no_omit(IAddForm, "exclude_from_nav")



# -*- coding: utf-8 -*-
from kitconcept.glossary.interfaces import IGlossary
from kitconcept.glossary.testing import INTEGRATION_TESTING
from plone import api
from plone.dexterity.interfaces import IDexterityFTI
from zope.component import createObject
from zope.component import queryUtility

import unittest


class GlossaryTypeTestCase(unittest.TestCase):

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]

        with api.env.adopt_roles(["Manager"]):
            self.g1 = api.content.create(self.portal, "Glossary", "g1")

    def test_adding(self):
        self.assertTrue(IGlossary.providedBy(self.g1))

    def test_fti(self):
        fti = queryUtility(IDexterityFTI, name="Glossary")
        self.assertIsNotNone(fti)

    def test_schema(self):
        fti = queryUtility(IDexterityFTI, name="Glossary")
        schema = fti.lookupSchema()
        self.assertEqual(IGlossary, schema)

    def test_factory(self):
        fti = queryUtility(IDexterityFTI, name="Glossary")
        factory = fti.factory
        new_object = createObject(factory)
        self.assertTrue(IGlossary.providedBy(new_object))

    def test_is_selectable_as_folder_default_view(self):
        self.portal.setDefaultPage("g1")
        self.assertEqual(self.portal.default_page, "g1")

    def test_not_allowed_content_types(self):
        from plone.api.exc import InvalidParameterError

        with self.assertRaises(InvalidParameterError):
            api.content.create(self.g1, "Document", "test")



# -*- coding: utf-8 -*-
from kitconcept.glossary.testing import INTEGRATION_TESTING
from plone import api
from plone.app.textfield.value import RichTextValue

import string
import unittest


class BaseViewTestCase(unittest.TestCase):

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]

        with api.env.adopt_roles(["Manager"]):
            self.g1 = api.content.create(
                self.portal,
                "Glossary",
                "g1",
                title="Glossary",
                description="Glossary Description",
            )
            self.t1 = api.content.create(
                self.g1,
                "GlossaryTerm",
                "t1",
                title="First Term",
                definition=RichTextValue(
                    "First Term Description", "text/html", "text/html"
                ),
            )
            self.t2 = api.content.create(
                self.g1,
                "GlossaryTerm",
                "t2",
                title="Second Term",
                definition=RichTextValue(
                    "Second Term Description", "text/html", "text/html"
                ),
            )
            self.d1 = api.content.create(
                self.portal,
                "Document",
                "d1",
                title="Document",
                description="Document Description",
            )


class TermViewTestCase(BaseViewTestCase):
    def setUp(self):
        super(TermViewTestCase, self).setUp()
        self.view = api.content.get_view(u"view", self.t1, self.request)

    def test_get_entry(self):
        expected = {
            "definition": "First Term Description",
            "variants": None,
            "term": "First Term",
        }
        self.assertEqual(self.view.get_entry(), expected)


class GlossaryViewTestCase(BaseViewTestCase):
    def setUp(self):
        super(GlossaryViewTestCase, self).setUp()
        self.view = api.content.get_view(u"view", self.g1, self.request)

    def test_view(self):
        html = self.view()
        self.assertIn('All', html)
        self.assertIn('[0-9]', html)
        for letter in string.ascii_uppercase:
            self.assertIn(letter, html)



# -*- coding: utf-8 -*-
from kitconcept.glossary.testing import INTEGRATION_TESTING
from plone.browserlayer.utils import registered_layers
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone import api

import unittest

try:
    from Products.CMFPlone.factory import _IMREALLYPLONE5

    _IMREALLYPLONE5  # noqa
except ImportError:
    PLONE_5 = False
else:
    PLONE_5 = True

try:
    from Products.CMFPlone.utils import get_installer
except ImportError:
    get_installer = None


class InstallTestCase(unittest.TestCase):

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        if get_installer:
            self.installer = get_installer(self.portal, self.layer['request'])
        else:
            self.installer = api.portal.get_tool('portal_quickinstaller')
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.installer.installProducts(['kitconcept.glossary'])

    def test_product_installed(self):
        self.assertTrue(self.installer.isProductInstalled(
            'kitconcept.glossary'))

    def test_addon_layer(self):
        layers = [l.getName() for l in registered_layers()]
        self.assertIn("IGlossaryLayer", layers)

    def test_add_glossary_permission(self):
        permission = "kitconcept.glossary: Add Glossary"
        roles = self.portal.rolesOfPermission(permission)
        roles = [r["name"] for r in roles if r["selected"]]
        expected = ["Contributor", "Manager", "Owner", "Site Administrator"]
        self.assertListEqual(roles, expected)

    def test_add_term_permission(self):
        permission = "kitconcept.glossary: Add Glossary Term"
        roles = self.portal.rolesOfPermission(permission)
        roles = [r["name"] for r in roles if r["selected"]]
        expected = ["Contributor", "Manager", "Owner", "Site Administrator"]
        self.assertListEqual(roles, expected)


class UninstallTestCase(unittest.TestCase):

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        if get_installer:
            self.installer = get_installer(self.portal, self.layer['request'])
        else:
            self.installer = api.portal.get_tool('portal_quickinstaller')
        roles_before = api.user.get_roles(TEST_USER_ID)
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.installer.uninstallProducts(['kitconcept.glossary'])
        setRoles(self.portal, TEST_USER_ID, roles_before)

    def test_product_uninstalled(self):
        self.assertFalse(
            self.installer.isProductInstalled('kitconcept.glossary')
        )

    # def test_browserlayer_removed(self):
    #     from kitconcept.glossary.interfaces import IGlossaryLayer
    #     from plone.browserlayer import utils

    #     self.assertNotIn(IGlossaryLayer, utils.registered_layers())



# -*- coding: utf-8 -*-
PROJECTNAME = "kitconcept.glossary"

# by default, all standard content types will be enabled
DEFAULT_ENABLED_CONTENT_TYPES = [
    "Collection",
    "Document",
    "File",
    "Image",
    "Link",
    "News Item",
]



# -*- coding: utf-8 -*-
"""Setup testing infrastructure.

For Plone 5 we need to install plone.app.contenttypes.
"""
from plone import api
from plone.app.robotframework.testing import AUTOLOGIN_LIBRARY_FIXTURE
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import PloneSandboxLayer
from plone.testing import z2

import pkg_resources


try:
    pkg_resources.get_distribution("plone.app.contenttypes")
except pkg_resources.DistributionNotFound:
    from plone.app.testing import PLONE_FIXTURE
else:
    from plone.app.contenttypes.testing import (
        PLONE_APP_CONTENTTYPES_FIXTURE as PLONE_FIXTURE,
    )

IS_PLONE_5 = api.env.plone_version().startswith("5")


class Fixture(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        import kitconcept.glossary

        self.loadZCML(package=kitconcept.glossary)

    def setUpPloneSite(self, portal):
        self.applyProfile(portal, "kitconcept.glossary:default")


FIXTURE = Fixture()

INTEGRATION_TESTING = IntegrationTesting(
    bases=(FIXTURE,), name="kitconcept.glossary:Integration"
)

FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(FIXTURE,), name="kitconcept.glossary:Functional"
)

ROBOT_TESTING = FunctionalTesting(
    bases=(FIXTURE, AUTOLOGIN_LIBRARY_FIXTURE, z2.ZSERVER_FIXTURE),
    name="kitconcept.glossary:Robot",
)



# -*- coding: utf-8 -*-

from plone import api

from plone.restapi.services import Service

from kitconcept.glossary.interfaces import IGlossarySettings

SERVICE_NAME = "@glossary_terms"


class GetGlossaryTerms(Service):
    def _error(self, status, type, message):
        self.request.response.setStatus(status)
        return {"error": {"type": type, "message": message}}

    def reply(self):
        brains = api.content.find(portal_type="GlossaryTerm")

        terms = [
            {
                "id": brain["id"],
                "title": brain["Title"],
                "terms": [brain["Title"]] + list(brain["variants"]),
                "definition": brain["definition"] or "",
                "url": brain.getURL(),
            }
            for brain in brains
        ]
        settings = {
            "enabled": api.portal.get_registry_record(
                name="enable_tooltip", interface=IGlossarySettings, default=False,
            ),
            "enabled_types": api.portal.get_registry_record(
                name="enabled_content_types", interface=IGlossarySettings, default=[],
            ),
        }

        return {
            "terms": terms,
            "settings": settings,
        }



# -*- coding: utf-8 -*-
from kitconcept.glossary.interfaces import IGlossaryTerm
from kitconcept.glossary.testing import INTEGRATION_TESTING
from plone import api
from plone.dexterity.interfaces import IDexterityFTI
from zope.component import createObject
from zope.component import queryUtility

import unittest


class GlossaryTypeTestCase(unittest.TestCase):

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]

        with api.env.adopt_roles(["Manager"]):
            self.g1 = api.content.create(self.portal, "Glossary", "g1")

        self.t1 = api.content.create(self.g1, "GlossaryTerm", "t1")

    def test_adding(self):
        self.assertTrue(IGlossaryTerm.providedBy(self.t1))

    def test_adding_outside_glossary(self):
        from plone.api.exc import InvalidParameterError

        with self.assertRaises(InvalidParameterError):
            api.content.create(self.portal, "GlossaryTerm", "test")

    def test_fti(self):
        fti = queryUtility(IDexterityFTI, name="GlossaryTerm")
        self.assertIsNotNone(fti)

    def test_schema(self):
        fti = queryUtility(IDexterityFTI, name="GlossaryTerm")
        schema = fti.lookupSchema()
        self.assertEqual(IGlossaryTerm, schema)

    def test_factory(self):
        fti = queryUtility(IDexterityFTI, name="GlossaryTerm")
        factory = fti.factory
        new_object = createObject(factory)
        self.assertTrue(IGlossaryTerm.providedBy(new_object))



# -*- coding: utf-8 -*-
from plone.app.vocabularies.types import ReallyUserFriendlyTypesVocabulary
from zope.schema.vocabulary import SimpleVocabulary


class PortalTypesVocabulary(ReallyUserFriendlyTypesVocabulary):
    """Inherit from plone.app.vocabularies.ReallyUserFriendlyTypes; and
    filter the results. We don't want Glossary or Term to be listed.
    """

    def __call__(self, context):
        items = super(PortalTypesVocabulary, self).__call__(context)
        items = [i for i in items if i.token not in ("Glossary", "Term")]
        return SimpleVocabulary(items)


PortalTypesVocabularyFactory = PortalTypesVocabulary()



# -*- coding: utf-8 -*-
from setuptools import find_packages
from setuptools import setup

version = "1.0.0"
description = "Plone add-on product to define a glossary."
long_description = (
    open("README.rst").read()
    + "\n\n"
    + open("CHANGES.rst").read()
)

setup(
    name="kitconcept.glossary",
    version=version,
    description=description,
    long_description=long_description,
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Web Environment",
        "Framework :: Plone",
        "Framework :: Plone :: 4.3",
        "Framework :: Plone :: 5.1",
        "Framework :: Plone :: 5.2",
        "Intended Audience :: Education",
        "Intended Audience :: End Users/Desktop",
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.7",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    keywords="plone glossary",
    author="kitconcept GmbH",
    author_email="info@kitconcept.com",
    url="https://github.com/kitconcept/kitconcept.glossary",
    license="GPL version 2",
    packages=find_packages("src"),
    package_dir={"": "src"},
    namespace_packages=["kitconcept"],
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        "plone.api",
        "plone.app.dexterity",
        "plone.app.registry",
        "plone.dexterity",
        "plone.namedfile",
        "plone.restapi",
        "Products.CMFPlone >=4.3",
        "Products.GenericSetup",
        "setuptools",
        "zope.globalrequest",
        "zope.i18nmessageid",
        "zope.interface",
        "zope.schema",
    ],
    extras_require={
        "test": [
            "AccessControl",
            "plone.app.robotframework",
            "plone.app.testing [robot]",
            "plone.browserlayer",
            "plone.registry",
            "plone.testing",
            "robotsuite",
            "zope.component",
        ]
    },
    entry_points="""
    [z3c.autoinclude.plugin]
    target = plone
    """,
)



# -*- coding: utf-8 -*-
from kitconcept.glossary.controlpanel import IGlossarySettings
from kitconcept.glossary.interfaces import IGlossaryLayer
from kitconcept.glossary.testing import INTEGRATION_TESTING
from plone import api
from plone.app.testing import logout
from plone.registry.interfaces import IRegistry
from zope.component import getUtility
from zope.interface import alsoProvides
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID

import unittest

try:
    from Products.CMFPlone.factory import _IMREALLYPLONE5

    _IMREALLYPLONE5  # noqa
except ImportError:
    PLONE_5 = False
else:
    PLONE_5 = True

try:
    from Products.CMFPlone.utils import get_installer
except ImportError:
    get_installer = None


class ControlPanelTestCase(unittest.TestCase):

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        alsoProvides(self.request, IGlossaryLayer)
        self.controlpanel = self.portal["portal_controlpanel"]

    def test_controlpanel_has_view(self):
        view = api.content.get_view(
            u"glossary-settings", self.portal, self.request)
        self.assertTrue(view())

    def test_controlpanel_view_is_protected(self):
        from AccessControl import Unauthorized

        logout()
        with self.assertRaises(Unauthorized):
            self.portal.restrictedTraverse("@@glossary-settings")

    def test_controlpanel_installed(self):
        actions = [a.getAction(self)["id"]
                   for a in self.controlpanel.listActions()]
        self.assertIn("glossary", actions)

    # def test_controlpanel_removed_on_uninstall(self):
    #     qi = self.portal["portal_quickinstaller"]

    #     with api.env.adopt_roles(["Manager"]):
    #         qi.uninstallProducts(products=[PROJECTNAME])

    #     actions = [a.getAction(self)["id"] for a in self.controlpanel.listActions()]
    #     self.assertNotIn("glossary", actions)


class RegistryTestCase(unittest.TestCase):

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.registry = getUtility(IRegistry)
        self.settings = self.registry.forInterface(IGlossarySettings)  # noqa: P001

    def test_enable_tooltip_record_in_registry(self):
        self.assertTrue(hasattr(self.settings, "enable_tooltip"))
        self.assertEqual(self.settings.enable_tooltip, True)

    def test_enabled_content_types_record_in_registry(self):
        from kitconcept.glossary.config import DEFAULT_ENABLED_CONTENT_TYPES

        self.assertTrue(hasattr(self.settings, "enabled_content_types"))
        self.assertEqual(
            self.settings.enabled_content_types, DEFAULT_ENABLED_CONTENT_TYPES
        )

    def test_records_removed_on_uninstall(self):
        if get_installer:
            self.installer = get_installer(self.portal, self.layer['request'])
        else:
            self.installer = api.portal.get_tool('portal_quickinstaller')

        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.installer.uninstallProducts(['kitconcept.glossary'])

        records = [
            IGlossarySettings.__identifier__ + ".enable_tooltip",
            IGlossarySettings.__identifier__ + ".enabled_content_types",
        ]

        for r in records:
            self.assertNotIn(r, self.registry)



# -*- coding: utf-8 -*-
from kitconcept.glossary.testing import IS_PLONE_5
from kitconcept.glossary.testing import ROBOT_TESTING
from plone.testing import layered

import os
import robotsuite
import unittest


dirname = os.path.dirname(__file__)
files = os.listdir(dirname)
tests = [f for f in files if f.startswith("test_") and f.endswith(".robot")]

# FIXME: skip RobotFramework tests in Plone 5
if IS_PLONE_5:
    tests = []


def test_suite():
    suite = unittest.TestSuite()
    suite.addTests(
        [
            layered(
                robotsuite.RobotTestSuite(t, noncritical=["Expected Failure"]),
                layer=ROBOT_TESTING,
            )
            for t in tests
        ]
    )
    return suite
